@@ -3,7 +3,9 @@ from attributes import *
 from writer import *
 from exceptions import *
 
-__all__ = ['Handle', 'Model',
-           'Attribute', 'IntegerField', 'ReferenceField', 'ListField', 'SetField', 'Recursive',
+__all__ = ['Handle', 'ishandle', 'Model',
+           'Attribute', 'BooleanField', 'IntegerField', 'FloatField',
+           'UTCDateTimeField', 'ReferenceField', 'ListField', 'SetField',
+           'Recursive',
            'ModelWriter', 'ListFieldWriter', 'SetFieldWriter',
            'Error', 'NotFoundError', 'UniqueError']
